@@ -6,24 +6,31 @@
 ----------------------------------------------------------------------
 -- |
 -- Module      :  Data.MemoTrie
--- Copyright   :  (c) Conal Elliott 2008
+-- Copyright   :  (c) Conal Elliott 2008-2012
 -- License     :  BSD3
 -- 
 -- Maintainer  :  conal@conal.net
 -- Stability   :  experimental
 -- 
 -- Trie-based memoizer
--- Adapted from sjanssen's paste: \"a lazy trie\" <http://hpaste.org/3839>.
+-- 
+-- Adapted from sjanssen's paste: \"a lazy trie\" <http://hpaste.org/3839>,
+-- which I think is based on Ralf Hinze's paper "Memo Functions,
+-- Polytypically!".
 ----------------------------------------------------------------------
 
 module Data.MemoTrie
-  ( HasTrie(..), domain, idTrie, (@.@)
+  ( HasTrie, (:->:)(..), trie, untrie, enumerate
+  , domain, idTrie, (@.@)
   -- , trie2, trie3, untrie2, untrie3
   , memo, memo2, memo3, mup
   , inTrie, inTrie2, inTrie3
   -- , untrieBits
   ) where
 
+-- Export the parts of HasTrie separately in order to get the associated data
+-- type constructors, so I can define instances of other classes on them.
+
 import Data.Bits
 import Data.Word
 import Data.Int
@@ -32,6 +39,8 @@ import Control.Arrow (first,(&&&))
 import Data.Monoid
 import Data.Function (on)
 
+import Data.Void
+
 -- import Prelude hiding (id,(.))
 -- import Control.Category
 -- import Control.Arrow
@@ -139,6 +148,13 @@ inTrie3 = untrie ~> inTrie2
 
 ---- Instances
 
+instance HasTrie Void where
+  -- As suggested by Audun
+  data Void :->: a = VoidTrie 
+  trie _ = VoidTrie
+  untrie VoidTrie = absurd
+  enumerate VoidTrie = []
+
 instance HasTrie () where
     newtype () :->: a = UnitTrie a
     trie f = UnitTrie (f ())
@@ -323,7 +339,7 @@ WordInstance(Word64,Word64Trie)
 
 
 -- | Extract bits in little-endian order
-bits :: Bits t => t -> [Bool]
+bits :: (Num t, Bits t) => t -> [Bool]
 bits 0 = []
 bits x = testBit x 0 : bits (shiftR x 1)
 
@@ -333,7 +349,7 @@ unbit False = 0
 unbit True  = 1
 
 -- | Bit list to value
-unbits :: Bits t => [Bool] -> t
+unbits :: (Num t, Bits t) => [Bool] -> t
 unbits [] = 0
 unbits (x:xs) = unbit x .|. shiftL (unbits xs) 1
 
@@ -371,19 +387,15 @@ instance HasTrie Integer where
     enumerate (IntegerTrie t) = enum' unbitsZ t
 
 
-unbitsZ :: (Bits n) => (Bool,[Bool]) -> n
+unbitsZ :: (Num n, Bits n) => (Bool,[Bool]) -> n
 unbitsZ (positive,bs) = sig (unbits bs)
  where
    sig | positive  = id
        | otherwise = negate
 
-bitsZ :: (Ord n, Bits n) => n -> (Bool,[Bool])
+bitsZ :: (Num n, Ord n, Bits n) => n -> (Bool,[Bool])
 bitsZ = (>= 0) &&& (bits . abs)
 
--- bitsZ n = (sign n, bits (abs n))
-
-
-
 -- TODO: make these definitions more systematic.
 
 
